/**@file ble_mgmt_c.h
 *
 * @defgroup ble_mgmt_c BLE Management Service Client
 * @{
 * @ingroup  ble_sdk_srv
 * @brief    BLE Management Service Client module.
 *
 * @details  This module contains the APIs and types exposed by the BLE Management Service Client
 *           module. These APIs and types can be used by the application to perform discovery of
 *           BLE Management Service at the peer and interact with it.
 *
 * @note    The application must register this module as BLE event observer using the
 *          NRF_SDH_BLE_OBSERVER macro. Example:
 *          @code
 *              ble_mgmt_c_t instance;
 *              NRF_SDH_BLE_OBSERVER(anything, BLE_MGMT_C_BLE_OBSERVER_PRIO,
 *                                   ble_mgmt_c_on_ble_evt, &instance);
 *          @endcode
 */

#ifndef BLE_MGMT_C_H__
#define BLE_MGMT_C_H__

#include <stdint.h>
#include "ble.h"
#include "ble_db_discovery.h"
#include "sdk_config.h"
#include "nrf_sdh_ble.h"
#include "nrf_queue.h"

#ifdef __cplusplus
extern "C" {
#endif

/**@brief   Macro for defining a ble_mgmt_c instance.
 *
 * @param   _name   Name of the instance.
 * @hideinitializer
 */
#define BLE_MGMT_C_DEF(_name)                                                                        \
static ble_mgmt_c_t _name;                                                                           \
NRF_SDH_BLE_OBSERVER(_name ## _obs,                                                                 \
                     BLE_MGMT_C_BLE_OBSERVER_PRIO,                                                   \
                     ble_mgmt_c_on_ble_evt, &_name)


  //! Auth service
#define BLE_FAS_BASE_UUID               0xFF60
#define BLE_FAS_UUID_RAND_NUM_CLR       0xFF61
#define BLE_FAS_UUID_CIPHOR_WRITE       0xFF62
#define BLE_FAS_BASE_UUID                      0xFF60
#define BLE_FAS_UUID_RANDOM_NUMBER_CLEAR       0xFF61
#define BLE_FAS_UUID_RANDOM_NUMBER_CIPHER      0xFF62
#define BLE_FAS_HDL_OLD_ID_RANDOM_NUMBER_CLEAR     19
#define BLE_FAS_HDL_OLD_ID_RANDOM_NUMBER_CIPHER    21
/**
 * @defgroup mgmt_c_enums Enumerations
 * @{
 */

/**@brief MGMT Client event type. */
typedef enum
{
    BLE_MGMT_C_EVT_DISCOVERY_COMPLETE = 1,  /**< Event indicating that the BLE Management Service has been discovered at the peer. */
    BLE_MGMT_C_EVT_WRITE_COMPLETE,
    BLE_MGMT_C_EVT_RANDOM_NUMBER_READ         /**< Event indicating that random number is available to calculate cipher value */
} ble_mgmt_c_evt_type_t;

/** @} */

/**
 * @defgroup mgmt_c_structs Structures
 * @{
 */


/**@brief Structure containing the handles related to the BLE Management Service found on the peer. */
typedef struct
{
    uint16_t mgmt_cccd_handle;  /**< Handle of the CCCD of the BLE Management characteristic. */
    uint16_t rnc_handle;       /**< Handle of the Random Number Clear characteristic. */
    uint16_t cvw_handle;       /**< Handle of the cipher value write characteristic. */
} mgmt_db_t;

/**@brief BLE Management Event structure. */
typedef struct
{
    ble_mgmt_c_evt_type_t evt_type;    /**< Type of the event. */
    uint16_t             conn_handle; /**< Connection handle on which the BLE Management service was discovered on the peer device..*/
    union
    {
        mgmt_db_t  peer_db;            /**< BLE Management related handles found on the peer device.. This will be filled if the evt_type is @ref BLE_MGMT_C_EVT_DISCOVERY_COMPLETE.*/
        uint16_t  rnc;                /**< Random Number read. This will be filled if the evt_type is @ref BLE_MGMT_C_EVT_RANDOM_NUMBER_READ. */
        uint16_t  cvw;                /**< cipher value to be sent. This will be filled if the evt_type is @ref BLE_MGMT_C_EVT_RANDOM_NUMBER_READ. */
    } params;
} ble_mgmt_c_evt_t;

/** @} */

/**
 * @defgroup mgmt_c_types Types
 * @{
 */

// Forward declaration of the ble_bas_t type.
typedef struct ble_mgmt_c_s ble_mgmt_c_t;

/**@brief   Event handler type.
 *
 * @details This is the type of the event handler that should be provided by the application
 *          of this module in order to receive events.
 */
typedef void (* ble_mgmt_c_evt_handler_t) (ble_mgmt_c_t * p_ble_mgmt_c, ble_mgmt_c_evt_t * p_evt);

/** @} */

/**
 * @addtogroup mgmt_c_structs
 * @{
 */

/**@brief BLE Management Client structure.
 */
struct ble_mgmt_c_s
{
    uint16_t                conn_handle;      /**< Connection handle as provided by the SoftDevice. */
    mgmt_db_t                peer_mgmt_db;      /**< Handles related to MGMT on the peer*/
    ble_mgmt_c_evt_handler_t evt_handler;      /**< Application event handler to be called when there is an event related to the ble management service. */
};

/**@brief BLE Management Client initialization structure.
 */
typedef struct
{
    ble_mgmt_c_evt_handler_t evt_handler;  /**< Event handler to be called by the BLE Management Client module whenever there is an event related to the BLE Management Service. */
} ble_mgmt_c_init_t;

/** @} */

/**
 * @defgroup mgmt_c_functions Functions
 * @{
 */

/**@brief     Function for initializing the ble management client module.
 *
 * @details   This function will register with the DB Discovery module. There it
 *            registers for the BLE Management Service. Doing so will make the DB Discovery
 *            module look for the presence of a BLE Management Service instance at the peer when a
 *            discovery is started.
 *
 * @param[in] p_ble_mgmt_c      Pointer to the ble management client structure.
 * @param[in] p_ble_mgmt_c_init Pointer to the ble management initialization structure containing the
 *                             initialization information.
 *
 * @retval    NRF_SUCCESS On successful initialization. Otherwise an error code. This function
 *                        propagates the error code returned by the Database Discovery module API
 *                        @ref ble_db_discovery_evt_register.
 */
uint32_t ble_mgmt_c_init(ble_mgmt_c_t * p_ble_mgmt_c, ble_mgmt_c_init_t * p_ble_mgmt_c_init);


/**@brief     Function for handling BLE events from the SoftDevice.
 *
 * @details   This function will handle the BLE events received from the SoftDevice. If a BLE
 *            event is relevant to the BLE Management Client module, then it uses it to update
 *            interval variables and, if necessary, send events to the application.
 *
 * @param[in] p_ble_evt     Pointer to the BLE event.
 * @param[in] p_context     Pointer to the ble management client structure.
 */
void ble_mgmt_c_on_ble_evt(ble_evt_t const * p_ble_evt, void * p_context);



/**@brief     Function for handling events from the database discovery module.
 *
 * @details   Call this function when getting a callback event from the DB discovery modue.
 *            This function will handle an event from the database discovery module, and determine
 *            if it relates to the discovery of ble management service at the peer. If so, it will
 *            call the application's event handler indicating that the ble management service has been
 *            discovered at the peer. It also populates the event with the service related
 *            information before providing it to the application.
 *
 * @param[in] p_ble_mgmt_c Pointer to the ble management client structure instance to associate.
 * @param[in] p_evt Pointer to the event received from the database discovery module.
 *
 */
void ble_mgmt_on_db_disc_evt(ble_mgmt_c_t * p_ble_mgmt_c, const ble_db_discovery_evt_t * p_evt);


/**@brief     Function for assigning a handles to a this instance of mgmt_c.
 *
 * @details   Call this function when a link has been established with a peer to
 *            associate this link to this instance of the module. This makes it
 *            possible to handle several link and associate each link to a particular
 *            instance of this module.The connection handle and attribute handles will be
 *            provided from the discovery event @ref BLE_MGMT_C_EVT_DISCOVERY_COMPLETE.
 *
 * @param[in] p_ble_mgmt_c        Pointer to the ble management client structure instance to associate.
 * @param[in] conn_handle        Connection handle to associated with the given BLE Management Client Instance.
 * @param[in] p_peer_mgmt_handles Attribute handles for the MGMT server you want this MGMT_C client to
 *                               interact with.
 */
uint32_t ble_mgmt_c_handles_assign(ble_mgmt_c_t *    p_ble_mgmt_c,
                                  uint16_t         conn_handle,
                                  const mgmt_db_t * p_peer_mgmt_handles);

/**@brief Function for creating a message for reading random number.
 */
uint32_t ble_mgmt_c_read_random_number(ble_mgmt_c_t * p_ble_mgmt_c);

/** @} */ // End tag for Function group.

/**@brief Function for creating a message for writing cipher value.
 */
uint32_t ble_mgmt_c_write_cipher_value(ble_mgmt_c_t * p_mgmt_c, ble_mgmt_c_evt_t * p_mgmt_c_evt);

/**@brief Function for creating a message for writing by ble handle.
 */
uint32_t ble_mgmt_char_write_by_handle(uint16_t conn_handle, uint16_t handle, uint8_t * data, uint8_t len);

/**@brief Function for creating a message for reading by ble handle.
 */
uint32_t ble_mgmt_char_read_by_handle(uint16_t conn_handle, uint16_t handle);

/**@brief     Function to validate(presence in predefined list) valid UUID/Handle as parameter.
 *
 * @details
 *
 * @param[in] param to be validated
 *
 * @return TRUE/FALSE
  */
bool ble_mgmt_is_valid_param(uint16_t param, uint16_t * p_param_list, uint8_t len);
/**@brief Function for creating a message for writing by ble handle.
 */
uint32_t ble_common_write_by_handle(uint16_t conn_handle, uint16_t handle, uint8_t * data, uint8_t len);
/**@brief Function for creating a message for reading random number.
 */
uint32_t ble_common_read_by_handle(uint16_t conn_handle, uint16_t handle);

#ifdef __cplusplus
}
#endif

#endif // BLE_MGMT_C_H__

/** @} */ // End tag for the file.
